#!/bin/bash
# Copyright 2017, CADennis, FreeCAD Community, 
# Licensed under Creative Commons BY
# ============================================================================
# The song might internationally evolve as follows:
#
# USA    :   (cdef)         (bc)---------(de)-------(fga)--------(bc)
#                  \       /   \           \           \           \
# master :         ( )   ( )   ( )   ( )   ( )   ( )   ( )   ( )   ( )
#                    \   /           /           /           /
# Germany:            (ga)-------(hcd)-------(efg)--------(ah)
# 
# Both get lazy from (ga) onwards. The master would have the effective states
#
# (cdef)->(cdefga)->(cdefgabc)->(cdefgahcd)->(cdefgabcde)->(cdefgahcdefg)->
# -> (cdefgabcdefga)->(cdefgahcdefgah)->(cdefgabcdefgabc)
#
# Other musicians, who care maybe only about the first octave and 
# try to stay synchronized, might get pretty annoyed and confused about 
# these frequent, repetitive overwrites of 
#
# h->b->h->b->h-> ...
# ============================================================================
mkdir torwaltz
cd torwaltz
mkdir protocol
mkdir torwaltz_git
cd torwaltz_git
git init
echo "" > song.txt
git add song.txt
git commit -m "let's make music"
# ====
git checkout -b usa
echo "cdef" >> song.txt
git add song.txt 
git commit -m "cdef - with love from USA"
# ====
git checkout master
git merge usa
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout -b germany
echo "ga" >> song.txt
git add song.txt 
git commit -m "ga - with love from Germany"
# ====
git checkout master
git merge germany
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout usa
echo "bc" >> song.txt
git add song.txt 
git commit -m "bc - with love from USA"
git rebase master 
sed -i '/<<<<<<</d' song.txt
sed -i '/>>>>>>>/d' song.txt
sed -i '/=======/d' song.txt
git add song.txt
git rebase --continue
# ====
git checkout master
git merge usa
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
echo "Both get lazy now"
# ====
git checkout germany
echo "hcd" >> song.txt
git add song.txt 
git commit -m "hcd - with love from Germany"
git diff germany..master > mychanges.patch
ls -lsa
git checkout master
git branch -D germany
git checkout -b germany
ls -lsa
patch -R -d . < mychanges.patch
rm mychanges.patch
git add song.txt
git commit -m "hcd - with love from Germany"
# ====
git checkout master
git merge germany
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout usa
echo "de" >> song.txt
git add song.txt 
git commit -m "de - with love from USA"
git diff usa..master > mychanges.patch
ls -lsa
git checkout master
git branch -D usa
git checkout -b usa
ls -lsa
patch -R -d . < mychanges.patch
rm mychanges.patch
git add song.txt
git commit -m "de - with love from USA"
# ====
git checkout master
git merge usa
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout germany
echo "efg" >> song.txt
git add song.txt 
git commit -m "efg - with love from Germany"
git diff germany..master > mychanges.patch
ls -lsa
git checkout master
git branch -D germany
git checkout -b germany
ls -lsa
patch -R -d . < mychanges.patch
rm mychanges.patch
git add song.txt
git commit -m "efg - with love from Germany"
# ====
git checkout master
git merge germany
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout usa
echo "fga" >> song.txt
git add song.txt 
git commit -m "fga - with love from USA"
git diff usa..master > mychanges.patch
ls -lsa
git checkout master
git branch -D usa
git checkout -b usa
ls -lsa
patch -R -d . < mychanges.patch
rm mychanges.patch
git add song.txt
git commit -m "fga - with love from USA"
# ====
git checkout master
git merge usa
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout germany
echo "ah" >> song.txt
git add song.txt 
git commit -m "ah - with love from Germany"
git diff germany..master > mychanges.patch
ls -lsa
git checkout master
git branch -D germany
git checkout -b germany
ls -lsa
patch -R -d . < mychanges.patch
rm mychanges.patch
git add song.txt
git commit -m "ah - with love from Germany"
# ====
git checkout master
git merge germany
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git checkout usa
echo "bc" >> song.txt
git add song.txt 
git commit -m "bc - with love from USA"
git diff usa..master > mychanges.patch
ls -lsa
git checkout master
git branch -D usa
git checkout -b usa
ls -lsa
patch -R -d . < mychanges.patch
rm mychanges.patch
git add song.txt
git commit -m "bc - with love from USA"
# ====
git checkout master
git merge usa
NOW=$(date +%Y_%m_%d-%H%M--%S.%3N)
SNAPSHOT=`git rev-parse --short HEAD`
cp song.txt ../protocol/master_snapshot-${NOW}_[${SNAPSHOT}]
# ====
git log --graph --all --oneline
git status
echo "=========="
echo "let's see HEAD (all 'b' - you wouldn't notice anything)"
cat song.txt
echo "=========="
echo "let's see HEAD~1 (all 'h' - oops, how is that possible?!)"
git checkout -q HEAD~1
cat song.txt
echo "=========="
echo "let's see HEAD~2 (all 'h' - why that, why no 'fga' from USA?!)"
git checkout -q HEAD~2
cat song.txt
echo "=========="
echo "let's see germany (all 'h' - consistent in himself, but not synced)"
git checkout germany
cat song.txt
echo "=========="
echo "If you dare, have a look at the toggling h->b->h->b->... in the protocoled effective master snapshots:"
cd ../protocol
pwd
ls -l
echo "=========="
echo "Any 3rd developer trying to sync with these two bad guys would have given up ;-)"

